<?php

namespace FiloBlu\PrevNext\Helper;

class Data extends \Magento\Framework\App\Helper\AbstractHelper {

    /**
     * Core registry
     *
     * @var \Magento\Framework\Registry
     */
    protected $_coreRegistry = null;
    protected $_productCollectionFactory;

    /**
     * Catalog layer
     *
     * @var \Magento\Catalog\Model\Layer
     */
    protected $_catalogLayer;
    protected $_productloader;
    protected $_categoryloader;

    public function __construct(
    \Magento\Framework\App\Helper\Context $context, \Magento\Framework\Registry $registry, \Magento\Catalog\Model\ResourceModel\Product\CollectionFactory $productCollectionFactory, \Magento\Catalog\Model\Layer\Resolver $layerResolver, \Magento\Catalog\Model\ProductFactory $_productloader, \Magento\Catalog\Model\CategoryFactory $_categoryloader
    ) {
        $this->_coreRegistry = $registry;
        $this->_productCollectionFactory = $productCollectionFactory;
        $this->_catalogLayer = $layerResolver->get();
        $this->_productloader = $_productloader;
        $this->_categoryloader = $_categoryloader;
        parent::__construct($context);
    }

    public function getConfig($config_path, $section = 'filoblu_prevnext_section') {
        return $this->scopeConfig->getValue(
                        $section . '/' . $config_path, \Magento\Store\Model\ScopeInterface::SCOPE_STORE
        );
    }

    public function getPrevNextAndCurrentCategoryLink($categoryId = null, $prodId = null) {

        if (!$prodId) {
            $prodId = $this->_coreRegistry->registry('current_product')->getId();
        }
        
        if (!$categoryId) {
            $currentCategory = $this->_coreRegistry->registry('current_category');
        } else {
            $currentCategory = $this->_categoryloader->create()->load($categoryId);
        }
        
        if ($currentCategory) {
            
            $currentProduct = $this->_productloader->create()->load($prodId);
            
            if (!in_array($currentCategory->getId(), $currentProduct->getCategoryIds())) {
                return false;
            }
            
            $categoryProducts = $this->_productCollectionFactory->create();
            $categoryProducts->addAttributeToSort('position');
            $categoryProducts->addAttributeToSelect('entity_id');
            $categoryProducts->addCategoryFilter($currentCategory);
            $categoryProducts->addAttributeToFilter('visibility', \Magento\Catalog\Model\Product\Visibility::VISIBILITY_BOTH);
            $categoryProducts->addAttributeToFilter('status', \Magento\Catalog\Model\Product\Attribute\Source\Status::STATUS_ENABLED);

            $this->_catalogLayer->prepareProductCollection($categoryProducts);


            $values = $categoryProducts->getColumnValues('entity_id');

            $key = array_search($prodId, $values);

            if (isset($values[$key - 1])) {
                $productIdPrev = $values[$key - 1];
            }

            if (isset($values[$key + 1])) {
                $productIdNext = $values[$key + 1];
            }

            $return = new \Magento\Framework\DataObject();


            if (isset($productIdPrev)) {
                $productPrev = $this->_productloader->create()->load($productIdPrev);
                $return->setPrevUrl($productPrev->getProductUrl());
            }

            if (isset($productIdNext)) {
                $productNext = $this->_productloader->create()->load($productIdNext);
                $return->setNextUrl($productNext->getProductUrl());
            }

            $return->setBackUrl($currentCategory->getUrl());

            return $return;
        }

        return false;
    }

}
